"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SlickDraggableGrouping = void 0;
const utils_1 = require("@slickgrid-universal/utils");
const bindingEvent_service_1 = require("../services/bindingEvent.service");
const domUtilities_1 = require("../services/domUtilities");
/**
 *
 * Draggable Grouping contributed by:  Muthukumar Selconstasu
 *  muthukumar{dot}se{at}gmail{dot}com
 *  github.com/muthukumarse/Slickgrid
 *
 * NOTES:
 *     This plugin provides the Draggable Grouping feature
 *
 * A plugin to add drop-down menus to column headers.
 * To specify a custom button in a column header, extend the column definition like so:
 *   this.columnDefinitions = [{
 *     id: 'cost', name: 'Cost', field: 'cost',
 *     grouping: {
 *       getter: 'cost',
 *       formatter: (g) => `Cost: ${g.value} <span style="color:green">(${g.count} items)</span>`,
 *       aggregators: [new Aggregators.Sum('cost')],
 *       aggregateCollapsed: true,
 *       collapsed: true
 *     }
 *   }];
 */
class SlickDraggableGrouping {
    /** Constructor of the SlickGrid 3rd party plugin, it can optionally receive options */
    constructor(extensionUtility, pubSubService, sharedService) {
        this.extensionUtility = extensionUtility;
        this.pubSubService = pubSubService;
        this.sharedService = sharedService;
        this._gridColumns = [];
        this._gridUid = '';
        this._subscriptions = [];
        this._defaults = {
            dropPlaceHolderText: 'Drop a column header here to group by the column',
            hideToggleAllButton: false,
            toggleAllButtonText: '',
            toggleAllPlaceholderText: 'Toggle all Groups',
        };
        this.columnsGroupBy = [];
        this.pluginName = 'DraggableGrouping';
        this._bindEventService = new bindingEvent_service_1.BindingEventService();
        this._eventHandler = new Slick.EventHandler();
        this.onGroupChanged = new Slick.Event();
    }
    get addonOptions() {
        return this._addonOptions;
    }
    /** Getter of SlickGrid DataView object */
    get dataView() {
        var _a, _b, _c;
        return (_c = (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.getData) === null || _b === void 0 ? void 0 : _b.call(_a)) !== null && _c !== void 0 ? _c : {};
    }
    get dropboxElement() {
        return this._dropboxElm;
    }
    get droppableInstance() {
        return this._droppableInstance;
    }
    get sortableInstance() {
        return this._sortableInstance;
    }
    get eventHandler() {
        return this._eventHandler;
    }
    get grid() {
        var _a, _b;
        return (_b = (_a = this._grid) !== null && _a !== void 0 ? _a : this.sharedService.slickGrid) !== null && _b !== void 0 ? _b : {};
    }
    get gridOptions() {
        var _a;
        return (_a = this.sharedService.gridOptions) !== null && _a !== void 0 ? _a : {};
    }
    /** Getter for the grid uid */
    get gridUid() {
        var _a, _b;
        return this._gridUid || ((_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.getUID()) !== null && _b !== void 0 ? _b : '');
    }
    get gridUidSelector() {
        return this.gridUid ? `#${this.gridUid}` : '';
    }
    /** Initialize plugin. */
    init(grid, groupingOptions) {
        var _a, _b, _c, _d, _f, _g;
        this._addonOptions = { ...this._defaults, ...groupingOptions };
        this._grid = grid;
        if (grid) {
            this._gridUid = grid.getUID();
            this._gridColumns = grid.getColumns();
            this._dropboxElm = grid.getPreHeaderPanel();
            // add optional group "Toggle All" with its button & text when provided
            if (!this._addonOptions.hideToggleAllButton) {
                this._groupToggler = (0, domUtilities_1.createDomElement)('div', {
                    className: 'slick-group-toggle-all',
                    title: (_a = this._addonOptions.toggleAllPlaceholderText) !== null && _a !== void 0 ? _a : '',
                    style: { display: 'none' },
                });
                const groupTogglerIconElm = (0, domUtilities_1.createDomElement)('span', { className: 'slick-group-toggle-all-icon' });
                this._groupToggler.appendChild(groupTogglerIconElm);
                if (this.gridOptions.enableTranslate && this._addonOptions.toggleAllButtonTextKey) {
                    this._addonOptions.toggleAllButtonText = this.extensionUtility.translateWhenEnabledAndServiceExist(this._addonOptions.toggleAllButtonTextKey, 'TEXT_TOGGLE_ALL_GROUPS');
                }
                if (this.gridOptions.enableTranslate && this._addonOptions.toggleAllPlaceholderTextKey) {
                    this._addonOptions.toggleAllPlaceholderText = this.extensionUtility.translateWhenEnabledAndServiceExist(this._addonOptions.toggleAllPlaceholderTextKey, 'TEXT_TOGGLE_ALL_GROUPS');
                }
                this._groupToggler.title = (_b = this._addonOptions.toggleAllPlaceholderText) !== null && _b !== void 0 ? _b : '';
                if (this._addonOptions.toggleAllButtonText) {
                    this._groupToggler.appendChild((0, domUtilities_1.createDomElement)('span', {
                        className: 'slick-group-toggle-all-text',
                        textContent: this._addonOptions.toggleAllButtonText || ''
                    }));
                }
                this._dropboxElm.appendChild(this._groupToggler);
                // when calling Expand/Collapse All Groups from Context Menu, we also need to inform this plugin as well of the action
                this._subscriptions.push(this.pubSubService.subscribe('onContextMenuCollapseAllGroups', () => this.toggleGroupToggler(groupTogglerIconElm, true, false)), this.pubSubService.subscribe('onContextMenuExpandAllGroups', () => this.toggleGroupToggler(groupTogglerIconElm, false, false)));
            }
            this._dropboxPlaceholderElm = (0, domUtilities_1.createDomElement)('div', { className: 'slick-draggable-dropbox-toggle-placeholder' });
            if (this.gridOptions.enableTranslate && ((_c = this._addonOptions) === null || _c === void 0 ? void 0 : _c.dropPlaceHolderTextKey)) {
                this._addonOptions.dropPlaceHolderText = this.extensionUtility.translateWhenEnabledAndServiceExist(this._addonOptions.dropPlaceHolderTextKey, 'TEXT_TOGGLE_ALL_GROUPS');
            }
            this._dropboxPlaceholderElm.textContent = (_g = (_f = (_d = this._addonOptions) === null || _d === void 0 ? void 0 : _d.dropPlaceHolderText) !== null && _f !== void 0 ? _f : this._defaults.dropPlaceHolderText) !== null && _g !== void 0 ? _g : '';
            this._dropboxElm.appendChild(this._dropboxPlaceholderElm);
            this.setupColumnDropbox();
            this._eventHandler.subscribe(grid.onHeaderCellRendered, (_e, args) => {
                var _a;
                const node = args.node;
                if (!(0, utils_1.isEmptyObject)((_a = args.column) === null || _a === void 0 ? void 0 : _a.grouping) && node) {
                    node.style.cursor = 'pointer'; // add the pointer cursor on each column title
                    // also optionally add an icon beside each column title that can be dragged
                    if (this._addonOptions.groupIconCssClass) {
                        const groupableIconElm = (0, domUtilities_1.createDomElement)('span', { className: 'slick-column-groupable' });
                        if (this._addonOptions.groupIconCssClass) {
                            groupableIconElm.classList.add(...this._addonOptions.groupIconCssClass.split(' '));
                        }
                        node.appendChild(groupableIconElm);
                    }
                }
            });
            // when calling Clear All Groups from Context Menu, we also need to inform this plugin as well of the action
            this._subscriptions.push(this.pubSubService.subscribe('onContextMenuClearGrouping', () => this.clearDroppedGroups()));
            for (const col of this._gridColumns) {
                const columnId = col.field;
                grid.updateColumnHeader(columnId);
            }
        }
        return this;
    }
    /** Dispose the plugin. */
    dispose() {
        this.onGroupChanged.unsubscribe();
        this._eventHandler.unsubscribeAll();
        this.pubSubService.unsubscribeAll(this._subscriptions);
        (0, domUtilities_1.emptyElement)(document.querySelector('.slick-preheader-panel'));
        this._grid = undefined;
    }
    clearDroppedGroups() {
        this.columnsGroupBy = [];
        this.updateGroupBy('clear-all');
        const allDroppedGroupingElms = this._dropboxElm.querySelectorAll('.slick-dropped-grouping');
        for (const groupElm of Array.from(allDroppedGroupingElms)) {
            const groupRemoveBtnElm = document.querySelector('.slick-groupby-remove');
            groupRemoveBtnElm === null || groupRemoveBtnElm === void 0 ? void 0 : groupRemoveBtnElm.remove();
            groupElm === null || groupElm === void 0 ? void 0 : groupElm.remove();
        }
        // show placeholder text & hide the "Toggle All" when that later feature is enabled
        this._dropboxPlaceholderElm.style.display = 'inline-block';
        if (this._groupToggler) {
            this._groupToggler.style.display = 'none';
        }
    }
    setColumns(cols) {
        this._gridColumns = cols;
    }
    setDroppedGroups(groupingInfo) {
        const groupingInfos = Array.isArray(groupingInfo) ? groupingInfo : [groupingInfo];
        this._dropboxPlaceholderElm.style.display = 'none';
        for (const groupInfo of groupingInfos) {
            const column = $(this.grid.getHeaderColumn(groupInfo));
            this.handleGroupByDrop(this._dropboxElm, column);
        }
    }
    setupColumnReorder(grid, $headers, _headerColumnWidthDiff, setColumns, setupColumnResize, columns, getColumnIndex, uid, trigger) {
        $headers.filter(':ui-sortable').sortable('destroy');
        const headerDraggableGroupByElm = grid.getPreHeaderPanel();
        return $headers.sortable({
            distance: 3,
            cursor: 'default',
            tolerance: 'intersection',
            helper: 'clone',
            placeholder: 'slick-sortable-placeholder ui-state-default slick-header-column',
            forcePlaceholderSize: true,
            appendTo: 'body',
            start: (_e, ui) => {
                $(ui.helper).addClass('slick-header-column-active');
                const placeholderElm = headerDraggableGroupByElm.querySelector('.slick-draggable-dropbox-toggle-placeholder');
                if (placeholderElm) {
                    placeholderElm.style.display = 'inline-block';
                }
                const droppedGroupingElm = headerDraggableGroupByElm.querySelector('.slick-dropped-grouping');
                if (droppedGroupingElm) {
                    droppedGroupingElm.style.display = 'none';
                }
            },
            beforeStop: (_e, ui) => {
                $(ui.helper).removeClass('slick-header-column-active');
                const hasDroppedColumn = headerDraggableGroupByElm.querySelectorAll('.slick-dropped-grouping').length;
                if (hasDroppedColumn > 0) {
                    const placeholderElm = headerDraggableGroupByElm.querySelector('.slick-draggable-dropbox-toggle-placeholder');
                    if (placeholderElm) {
                        placeholderElm.style.display = 'none';
                    }
                    const droppedGroupingElm = headerDraggableGroupByElm.querySelector('.slick-dropped-grouping');
                    if (droppedGroupingElm) {
                        droppedGroupingElm.style.display = 'inline-block';
                    }
                }
            },
            stop: (e) => {
                if (!grid.getEditorLock().commitCurrentEdit()) {
                    $(e.target).sortable('cancel');
                    return;
                }
                const reorderedIds = $headers.sortable('toArray');
                // If frozen columns are used, headers has more than one entry and we need the ids from all of them.
                // though there is only really a left and right header, this will work even if that should change.
                if ($headers.length > 1) {
                    for (let headerIdx = 1, ln = $headers.length; headerIdx < ln; headerIdx += 1) {
                        const $header = $($headers[headerIdx]);
                        const ids = $header.sortable('toArray');
                        // Note: the loop below could be simplified with:
                        // reorderedIds.push.apply(reorderedIds,ids);
                        // However, the loop is more in keeping with way-backward compatibility
                        for (const id of ids) {
                            reorderedIds.push(id);
                        }
                    }
                }
                const reorderedColumns = [];
                for (const reorderedId of reorderedIds) {
                    reorderedColumns.push(columns[getColumnIndex(reorderedId.replace(uid, ''))]);
                }
                setColumns(reorderedColumns);
                trigger(grid.onColumnsReordered, { grid: this.grid });
                e.stopPropagation();
                setupColumnResize();
            }
        });
    }
    //
    // protected functions
    // ------------------
    addColumnGroupBy(column) {
        this.columnsGroupBy.push(column);
        this.updateGroupBy('add-group');
    }
    addGroupByRemoveClickHandler(id, _container, headerColumn, entry) {
        const groupRemoveElm = document.querySelector(`${this.gridUidSelector}_${id}_entry > .slick-groupby-remove`);
        if (groupRemoveElm) {
            this._bindEventService.bind(groupRemoveElm, 'click', () => {
                const boundedElms = this._bindEventService.boundedEvents.filter(boundedEvent => boundedEvent.element === groupRemoveElm);
                for (const boundedEvent of boundedElms) {
                    this._bindEventService.unbind(boundedEvent.element, 'click', boundedEvent.listener);
                }
                this.removeGroupBy(id, headerColumn, entry);
            });
        }
    }
    handleGroupByDrop(container, headerColumn) {
        const columnid = headerColumn.attr('id').replace(this._gridUid, '');
        let columnAllowed = true;
        this.columnsGroupBy.forEach(col => {
            if (col.id === columnid) {
                columnAllowed = false;
            }
        });
        if (columnAllowed) {
            this._gridColumns.forEach(col => {
                var _a;
                if (col.id === columnid) {
                    if (col.grouping !== null && !(0, utils_1.isEmptyObject)(col.grouping)) {
                        const columnName = headerColumn.children('.slick-column-name').first();
                        const entryElm = (0, domUtilities_1.createDomElement)('div', { id: `${this._gridUid}_${col.id}_entry`, className: 'slick-dropped-grouping', dataset: { id: `${col.id}` } });
                        const groupTextElm = (0, domUtilities_1.createDomElement)('div', {
                            textContent: columnName.length ? columnName.text() : headerColumn.text(),
                            style: { display: 'inline-flex' },
                        });
                        entryElm.appendChild(groupTextElm);
                        const groupRemoveIconElm = (0, domUtilities_1.createDomElement)('div', { className: 'slick-groupby-remove' });
                        if (this._addonOptions.deleteIconCssClass) {
                            groupRemoveIconElm.classList.add(...this._addonOptions.deleteIconCssClass.split(' '));
                        }
                        if (!this._addonOptions.deleteIconCssClass) {
                            groupRemoveIconElm.classList.add('slick-groupby-remove-image');
                        }
                        entryElm.appendChild(groupRemoveIconElm);
                        entryElm.appendChild(document.createElement('div'));
                        container.appendChild(entryElm);
                        // if we're grouping by only 1 group, at the root, we'll analyze Toggle All and add collapsed/expanded class
                        if (this._groupToggler && this.columnsGroupBy.length === 0) {
                            const togglerIcon = this._groupToggler.querySelector('.slick-group-toggle-all-icon');
                            if ((_a = col.grouping) === null || _a === void 0 ? void 0 : _a.collapsed) {
                                togglerIcon === null || togglerIcon === void 0 ? void 0 : togglerIcon.classList.add('collapsed');
                                togglerIcon === null || togglerIcon === void 0 ? void 0 : togglerIcon.classList.remove('expanded');
                            }
                            else {
                                togglerIcon === null || togglerIcon === void 0 ? void 0 : togglerIcon.classList.add('expanded');
                                togglerIcon === null || togglerIcon === void 0 ? void 0 : togglerIcon.classList.remove('collapsed');
                            }
                        }
                        this.addColumnGroupBy(col);
                        this.addGroupByRemoveClickHandler(col.id, container, headerColumn, entryElm);
                    }
                }
            });
            // show the "Toggle All" when feature is enabled
            if (this._groupToggler && this.columnsGroupBy.length > 0) {
                this._groupToggler.style.display = 'inline-block';
            }
        }
    }
    removeFromArray(arrayToModify, itemToRemove) {
        if (Array.isArray(arrayToModify)) {
            const itemIdx = arrayToModify.findIndex(a => a.id === itemToRemove.id);
            if (itemIdx >= 0) {
                arrayToModify.splice(itemIdx, 1);
            }
        }
        return arrayToModify;
    }
    removeGroupBy(id, _hdrColumn, entry) {
        entry.remove();
        const groupByColumns = [];
        this._gridColumns.forEach(col => groupByColumns[col.id] = col);
        this.removeFromArray(this.columnsGroupBy, groupByColumns[id]);
        if (this.columnsGroupBy.length === 0) {
            // show placeholder text & hide the "Toggle All" when that later feature is enabled
            this._dropboxPlaceholderElm.style.display = 'inline-block';
            if (this._groupToggler) {
                this._groupToggler.style.display = 'none';
            }
        }
        this.updateGroupBy('remove-group');
    }
    setupColumnDropbox() {
        this._droppableInstance = $(this._dropboxElm).droppable({
            activeClass: 'ui-state-default',
            hoverClass: 'ui-state-hover',
            accept: ':not(.ui-sortable-helper)',
            deactivate: () => {
                this._dropboxElm.classList.remove('slick-header-column-denied');
            },
            drop: (e, ui) => {
                this.handleGroupByDrop(e.target, ui.draggable);
            },
            over: (_e, ui) => {
                const id = (ui.draggable).attr('id').replace(this._gridUid, '');
                for (const col of this._gridColumns) {
                    if (col.id === id && !col.grouping) {
                        this._dropboxElm.classList.add('slick-header-column-denied');
                    }
                }
            }
        });
        this._sortableInstance = $(this._dropboxElm).sortable({
            items: 'div.slick-dropped-grouping',
            cursor: 'default',
            tolerance: 'pointer',
            helper: 'clone',
            update: (event) => {
                const sortArray = $(event.target).sortable('toArray', { attribute: 'data-id' });
                const newGroupingOrder = [];
                for (const sortGroupId of sortArray) {
                    for (const groupByColumn of this.columnsGroupBy) {
                        if (groupByColumn.id === sortGroupId) {
                            newGroupingOrder.push(groupByColumn);
                            break;
                        }
                    }
                }
                this.columnsGroupBy = newGroupingOrder;
                this.updateGroupBy('sort-group');
            }
        });
        if (this._groupToggler) {
            this._bindEventService.bind(this._groupToggler, 'click', ((event) => {
                const target = event.target.classList.contains('slick-group-toggle-all-icon') ? event.target : event.currentTarget.querySelector('.slick-group-toggle-all-icon');
                this.toggleGroupToggler(target, target === null || target === void 0 ? void 0 : target.classList.contains('expanded'));
            }));
        }
    }
    toggleGroupToggler(targetElm, collapsing = true, shouldExecuteDataViewCommand = true) {
        if (targetElm) {
            if (collapsing === true) {
                targetElm.classList.add('collapsed');
                targetElm.classList.remove('expanded');
                if (shouldExecuteDataViewCommand) {
                    this.dataView.collapseAllGroups();
                }
            }
            else {
                targetElm.classList.remove('collapsed');
                targetElm.classList.add('expanded');
                if (shouldExecuteDataViewCommand) {
                    this.dataView.expandAllGroups();
                }
            }
        }
    }
    updateGroupBy(originator) {
        if (this.columnsGroupBy.length === 0) {
            this.dataView.setGrouping([]);
            this._dropboxPlaceholderElm.style.display = 'inline-block';
            this.onGroupChanged.notify({ caller: originator, groupColumns: [] });
            return;
        }
        const groupingArray = [];
        this.columnsGroupBy.forEach(element => groupingArray.push(element.grouping));
        this.dataView.setGrouping(groupingArray);
        this._dropboxPlaceholderElm.style.display = 'none';
        this.onGroupChanged.notify({ caller: originator, groupColumns: groupingArray });
    }
}
exports.SlickDraggableGrouping = SlickDraggableGrouping;
//# sourceMappingURL=slickDraggableGrouping.js.map